@@ -103,9 +103,7 @@ static void dump_memory(hd_data_t *hd_data, memory_range_t *mem, int sparse, cha
 static void get_pnp_support_status(memory_range_t *mem, bios_info_t *bt);
 static void smbios_get_info(hd_data_t *hd_data, memory_range_t *mem, bios_info_t *bt);
 static void get_fsc_info(hd_data_t *hd_data, memory_range_t *mem, bios_info_t *bt);
-#ifndef LIBHD_TINY
 static void add_panel_info(hd_data_t *hd_data, bios_info_t *bt);
-#endif
 static void add_mouse_info(hd_data_t *hd_data, bios_info_t *bt);
 static void chk_vbox(hd_data_t *hd_data);
 static unsigned char crc(unsigned char *mem, unsigned len);
@@ -168,12 +166,10 @@ void hd_scan_bios(hd_data_t *hd_data)
   unsigned u, u1;
   memory_range_t mem;
   unsigned smp_ok;
-#ifndef LIBHD_TINY
   vbe_info_t *vbe;
   vbe_mode_info_t *mi;
   hd_res_t *res;
   str_list_t *sl, *sl0;
-#endif
 
   if(!hd_probe_feature(hd_data, pr_bios)) return;
 
@@ -367,9 +363,7 @@ void hd_scan_bios(hd_data_t *hd_data)
     get_pnp_support_status(&hd_data->bios_rom, bt);
     smbios_get_info(hd_data, &hd_data->bios_rom, bt);
     get_fsc_info(hd_data, &hd_data->bios_rom, bt);
-#ifndef LIBHD_TINY
     add_panel_info(hd_data, bt);
-#endif
     add_mouse_info(hd_data, bt);
     chk_vbox(hd_data);
   }
@@ -439,7 +433,6 @@ void hd_scan_bios(hd_data_t *hd_data)
     }
   }
 
-#ifndef LIBHD_TINY
   if(hd_probe_feature(hd_data, pr_bios_vesa)) {
     PROGRESS(4, 0, "vbe");
 
@@ -454,7 +447,9 @@ void hd_scan_bios(hd_data_t *hd_data)
       }
     }
 
+#if defined(__i386__) || defined (__x86_64__)
     get_vbe_info(hd_data, vbe);
+#endif
 
     if(vbe->ok) {
       bt->vbe_ver = vbe->version;
@@ -467,12 +462,6 @@ void hd_scan_bios(hd_data_t *hd_data)
 
       hd_set_hw_class(hd, hw_vbe);
 
-#if 0
-      hd->detail = new_mem(sizeof *hd->detail);
-      hd->detail->type = hd_detail_bios;
-      hd->detail->bios.data = bt = new_mem(sizeof *bt);
-#endif
-
       hd->vendor.name = new_str(vbe->vendor_name);
       hd->device.name = new_str(vbe->product_name);
       hd->sub_vendor.name = new_str(vbe->oem_name);
@@ -507,25 +496,8 @@ void hd_scan_bios(hd_data_t *hd_data)
           }
         }
       }
-
-#if 0
-      if(
-        hd->vend_name &&
-        !strcmp(hd->vend_name, "Matrox") &&
-        hd->device.name &&
-        (
-          strstr(hd->dev_name, "G200") ||
-          strstr(hd->dev_name, "G400") ||
-          strstr(hd->dev_name, "G450")
-        )
-      ) {
-        hd->broken = 1;
-      }
-#endif
-
     }
   }
-#endif	/* LIBHD_TINY */
 
   PROGRESS(5, 0, "32");
 
@@ -571,12 +543,6 @@ void read_memory(hd_data_t *hd_data, memory_range_t *mem)
   char *s = getenv("LIBHD_MEM");
 #endif
 
-#ifdef LIBHD_MEMCHECK
-  {
-    if(libhd_log) fprintf(libhd_log, ">%p\n", CALLED_FROM(read_memory, mem));
-  }
-#endif
-
   if(mem->data) free_mem(mem->data);
   mem->data = new_mem(mem->size);
 #ifdef BIOS_TEST
@@ -584,12 +550,6 @@ void read_memory(hd_data_t *hd_data, memory_range_t *mem)
 #else
   hd_read_mmap(hd_data, DEV_MEM, mem->data, mem->start, mem->size);
 #endif
-
-#ifdef LIBHD_MEMCHECK
-  {
-    if(libhd_log) fprintf(libhd_log, "<%p\n", CALLED_FROM(read_memory, mem));
-  }
-#endif
 }
 
 
@@ -608,7 +568,7 @@ void dump_memory(hd_data_t *hd_data, memory_range_t *mem, int sparse, char *labe
   ADD2LOG("----- %s 0x%05x - 0x%05x -----\n", label, mem->start, mem->start + mem->size - 1);
   for(u = 0; u < mem->size; u += step) {
     ADD2LOG("  %03x  ", u + mem->start);
-    hexdump(&hd_data->log, 1, mem->size - u > 0x10 ? 0x10 : mem->size - u, mem->data + u);
+    hd_log_hex(hd_data, 1, mem->size - u > 0x10 ? 0x10 : mem->size - u, mem->data + u);
     ADD2LOG("\n");
   }
   ADD2LOG("----- %s end -----\n", label);
@@ -719,7 +679,7 @@ void smbios_get_info(hd_data_t *hd_data, memory_range_t *mem, bios_info_t *bt)
     memcpy(sm->any.data, memory.data + ofs, slen);
     sm->any.handle = memory.data[ofs + 2] + (memory.data[ofs + 3] << 8);
     ADD2LOG("  type 0x%02x [0x%04x]: ", type, sm->any.handle);
-    if(slen) hexdump(&hd_data->log, 0, slen, sm->any.data);
+    if(slen) hd_log_hex(hd_data, 0, slen, sm->any.data);
     ADD2LOG("\n");
     if(type == sm_end) break;
     ofs += slen;
@@ -840,7 +800,6 @@ void get_fsc_info(hd_data_t *hd_data, memory_range_t *mem, bios_info_t *bt)
 }
 
 
-#ifndef LIBHD_TINY
 void add_panel_info(hd_data_t *hd_data, bios_info_t *bt)
 {
   unsigned width, height, xsize = 0, ysize = 0;
@@ -895,7 +854,7 @@ void add_panel_info(hd_data_t *hd_data, bios_info_t *bt)
     }
   }
 }
-#endif
+
 
 void add_mouse_info(hd_data_t *hd_data, bios_info_t *bt)
 {
@@ -1062,7 +1021,7 @@ void parse_mpconfig(hd_data_t *hd_data, memory_range_t *mem, smp_info_t *smp)
       len = type == 0 ? 20 : type <= 4 ? 8 : 16;
       ADD2LOG("  %stype %u, len %u\n    ", type > 4 ? "unknown ": "", type, len);
       if(len + u > cfg_len) len = cfg_len - u;
-      hexdump(&hd_data->log, 1, len, mem->data + u);
+      hd_log_hex(hd_data, 1, len, mem->data + u);
       ADD2LOG("\n");
       if(type > 4) break;
       if(type == 0) {
@@ -1082,7 +1041,7 @@ void parse_mpconfig(hd_data_t *hd_data, memory_range_t *mem, smp_info_t *smp)
       len = mem->data[u + cfg_len + 1];
       ADD2LOG("  type %u, len %u\n    ", type, len);
       if(len + u > xcfg_len) len = xcfg_len - u;
-      hexdump(&hd_data->log, 1, len, mem->data + cfg_len + u);
+      hd_log_hex(hd_data, 1, len, mem->data + cfg_len + u);
       ADD2LOG("\n");
       if(len < 2) {
         ADD2LOG("  oops: invalid record lenght\n");
