@@ -89,14 +89,14 @@ void read_cpuinfo(hd_data_t *hd_data)
   str_list_t *sl;
 
 #if defined(__i386__) || defined (__x86_64__)
-  char model_id[80], vendor_id[80], features[0x100];
+  char model_id[80], vendor_id[80], features[0x400];
   unsigned mhz, cache, family, model, stepping;
   double bogo;
   char *t0, *t;
 #endif
 
 #ifdef __ia64__
-  char model_id[0x100], vendor_id[80], features[0x100];
+  char model_id[0x100], vendor_id[80], features[0x400];
   unsigned mhz, family, model, stepping;
   double bogo;
 #endif
@@ -107,6 +107,22 @@ void read_cpuinfo(hd_data_t *hd_data)
   cpu_info_t *ct1;
 #endif
 
+#ifdef __arm__
+  char model_id[80], system_id[80], serial_number[80], features[0x400];
+  unsigned cpu_variation, cpu_revision, vendor_id;
+  unsigned u;
+  double bogo;
+  char *t0, *t;
+#endif
+
+#ifdef __aarch64__
+  char model_id[80], system_id[80], serial_number[80], features[0x400];
+  unsigned cpu_variation, cpu_revision, vendor_id;
+  unsigned u;
+  double bogo;
+  char *t0, *t;
+#endif
+
 #ifdef __PPC__
   char model_id[80], vendor_id[80], motherboard[80];
   unsigned mhz, cache, family, model, stepping;
@@ -126,6 +142,12 @@ void read_cpuinfo(hd_data_t *hd_data)
   unsigned u0, u1, u2, u3;
 #endif
 
+#ifdef __m68k__
+  char cpu[32], mmu[32], fpu[32];
+  unsigned mhz;
+  double bogo;
+#endif
+
   hd_data->cpu = read_file(PROC_CPUINFO, 0, 0);
   if((hd_data->debug & HD_DEB_CPU)) dump_cpu_data(hd_data);
   if(!hd_data->cpu) return;
@@ -189,6 +211,135 @@ void read_cpuinfo(hd_data_t *hd_data)
   }
 #endif	/* __alpha__ */
 
+#ifdef __arm__
+  *model_id = *system_id = *serial_number = 0;
+  cpu_variation = cpu_revision = 0;
+  ct = 0; bogo = 0;
+
+  hd_data->boot = boot_uboot;
+
+  for(sl = hd_data->cpu; sl; sl = sl->next) {
+    if(sscanf(sl->str, "Processor       : %79[^\n]", model_id) == 1) continue;
+    if(sscanf(sl->str, "Hardware        : %79[^\n]", system_id) == 1) continue;
+    if(sscanf(sl->str, "Features        : %1000[^\n]", features) == 1) continue;
+    if(sscanf(sl->str, "CPU variant     : 0x%x", &cpu_variation) == 1) continue;
+    if(sscanf(sl->str, "CPU implementer : 0x%x", &vendor_id) == 1) continue;
+    if(sscanf(sl->str, "CPU revision    : %d", &cpu_revision) == 1) continue;
+    if(!bogo && sscanf(sl->str, "BogoMIPS        : %lg", &bogo) == 1);
+    if(strstr(sl->str, "processor\t: ") == sl->str) cpus++;
+  }
+
+  /* if we didn't find any cpus, we might be running under qemu.
+     So simulate just one CPU to make the rest happy. */
+  if(!cpus) cpus = 1;
+  for(u = 0; u < cpus; u++) {
+    ct = new_mem(sizeof *ct);
+    ct->architecture = arch_arm;
+    ct->family = cpu_variation;
+    ct->model = cpu_revision;
+    ct->stepping = 0;
+    ct->cache = 0;
+    ct->clock = 0;
+    ct->bogo = bogo;
+    ct->platform = 0;
+    if(*system_id) ct->platform = new_str(system_id);
+    if(*model_id) ct->model_name = new_str(model_id);
+
+    switch (vendor_id) {
+    case 0x41: ct->vend_name = new_str("ARM Limited"); break;
+    case 0x44: ct->vend_name = new_str("Digital Equipment Corporation"); break;
+    case 0x4d: ct->vend_name = new_str("Motorola, Freescale Semiconductor Inc."); break;
+    case 0x51: ct->vend_name = new_str("QUALCOMM Inc."); break;
+    case 0x56: ct->vend_name = new_str("Marvell Semiconductor Inc."); break;
+    case 0x69: ct->vend_name = new_str("Intel Corporation."); break;
+    default:
+     {
+       char buf[80];
+       sprintf(buf, "unknown (%x)", vendor_id);
+       ct->vend_name = new_str(buf);
+     }
+    }
+
+    if(*features) {
+      for(t0 = features; (t = strsep(&t0, " ")); ) {
+        add_str_list(&ct->features, new_str(t));
+      }
+    }
+
+    hd = add_hd_entry(hd_data, __LINE__, 0);
+    hd->base_class.id = bc_internal;
+    hd->sub_class.id = sc_int_cpu;
+    hd->slot = u;
+    hd->detail = new_mem(sizeof *hd->detail);
+    hd->detail->type = hd_detail_cpu;
+    hd->detail->cpu.data = ct;
+  }
+#endif	/* __arm__ */
+
+#ifdef __aarch64__
+  *model_id = *system_id = *serial_number = 0;
+  cpu_variation = cpu_revision = 0;
+  ct = 0; bogo = 0;
+
+  hd_data->boot = boot_uboot;
+
+  for(sl = hd_data->cpu; sl; sl = sl->next) {
+    if(sscanf(sl->str, "Processor       : %79[^\n]", model_id) == 1) continue;
+    if(sscanf(sl->str, "Hardware        : %79[^\n]", system_id) == 1) continue;
+    if(sscanf(sl->str, "Features        : %1000[^\n]", features) == 1) continue;
+    if(sscanf(sl->str, "CPU variant     : 0x%x", &cpu_variation) == 1) continue;
+    if(sscanf(sl->str, "CPU implementer : 0x%x", &vendor_id) == 1) continue;
+    if(sscanf(sl->str, "CPU revision    : %d", &cpu_revision) == 1) continue;
+    if(!bogo && sscanf(sl->str, "BogoMIPS        : %lg", &bogo) == 1);
+    if(strstr(sl->str, "processor\t: ") == sl->str) cpus++;
+  }
+
+  /* if we didn't find any cpus, we might be running under qemu.
+     So simulate just one CPU to make the rest happy. */
+  if(!cpus) cpus = 1;
+  for(u = 0; u < cpus; u++) {
+    ct = new_mem(sizeof *ct);
+    ct->architecture = arch_aarch64;
+    ct->family = cpu_variation;
+    ct->model = cpu_revision;
+    ct->stepping = 0;
+    ct->cache = 0;
+    ct->clock = 0;
+    ct->bogo = bogo;
+    ct->platform = 0;
+    if(*system_id) ct->platform = new_str(system_id);
+    if(*model_id) ct->model_name = new_str(model_id);
+
+    switch (vendor_id) {
+    case 0x41: ct->vend_name = new_str("ARM Limited"); break;
+    case 0x44: ct->vend_name = new_str("Digital Equipment Corporation"); break;
+    case 0x4d: ct->vend_name = new_str("Motorola, Freescale Semiconductor Inc."); break;
+    case 0x51: ct->vend_name = new_str("QUALCOMM Inc."); break;
+    case 0x56: ct->vend_name = new_str("Marvell Semiconductor Inc."); break;
+    case 0x69: ct->vend_name = new_str("Intel Corporation."); break;
+    default:
+     {
+       char buf[80];
+       sprintf(buf, "unknown (%x)", vendor_id);
+       ct->vend_name = new_str(buf);
+     }
+    }
+
+    if(*features) {
+      for(t0 = features; (t = strsep(&t0, " ")); ) {
+        add_str_list(&ct->features, t);
+      }
+    }
+
+    hd = add_hd_entry(hd_data, __LINE__, 0);
+    hd->base_class.id = bc_internal;
+    hd->sub_class.id = sc_int_cpu;
+    hd->slot = u;
+    hd->detail = new_mem(sizeof *hd->detail);
+    hd->detail->type = hd_detail_cpu;
+    hd->detail->cpu.data = ct;
+  }
+#endif	/* __aarch64__ */
 
 #ifdef __sparc__
   *cpu_id = *fpu_id = *promlib = *prom = *type = *mmu = 0;
@@ -241,7 +392,7 @@ void read_cpuinfo(hd_data_t *hd_data)
   for(sl = hd_data->cpu; sl; sl = sl->next) {
     if(sscanf(sl->str, "model name : %79[^\n]", model_id) == 1);
     if(sscanf(sl->str, "vendor_id : %79[^\n]", vendor_id) == 1);
-    if(sscanf(sl->str, "flags : %255[^\n]", features) == 1);
+    if(sscanf(sl->str, "flags : %1023[^\n]", features) == 1);
     if(sscanf(sl->str, "bogomips : %lg", &bogo) == 1);
     if(sscanf(sl->str, "cpu MHz : %u", &mhz) == 1);
     if(sscanf(sl->str, "cache size : %u KB", &cache) == 1);
@@ -389,7 +540,7 @@ void read_cpuinfo(hd_data_t *hd_data)
   for(sl = hd_data->cpu; sl; sl = sl->next) {
     if(sscanf(sl->str, "model name : %255[^\n]", model_id) == 1);
     if(sscanf(sl->str, "vendor : %79[^\n]", vendor_id) == 1);
-    if(sscanf(sl->str, "features : %255[^\n]", features) == 1);
+    if(sscanf(sl->str, "features : %1023[^\n]", features) == 1);
     if(sscanf(sl->str, "cpu MHz : %u", &mhz) == 1);
     if(sscanf(sl->str, "family : %u", &family) == 1);
     if(sscanf(sl->str, "model : %u", &model) == 1);
@@ -496,6 +647,36 @@ void read_cpuinfo(hd_data_t *hd_data)
     }
   }
 #endif /* defined(__s390__) || defined(__s390x__) */
+
+#ifdef __m68k__
+  *cpu = *mmu = *fpu = 0;
+  mhz = 0;
+  bogo = 0;
+
+  for (sl = hd_data->cpu; sl; sl = sl->next) {
+    if (sscanf(sl->str, "CPU:            %31[^\n]", cpu) == 1);
+    if (sscanf(sl->str, "MMU:            %31[^\n]", mmu) == 1);
+    if (sscanf(sl->str, "FPU:            %31[^\n]", fpu) == 1);
+    if (sscanf(sl->str, "Clocking:       %u", &mhz) == 1);
+    if (sscanf(sl->str, "BogoMips:       %lg", &bogo) == 1);
+  }
+
+  if (*cpu) {
+    ct = new_mem(sizeof *ct);
+    ct->architecture = arch_68k;
+    ct->model_name = new_str(cpu);
+    ct->clock = mhz;
+    ct->bogo = bogo;
+
+    hd = add_hd_entry(hd_data, __LINE__, 0);
+    hd->base_class.id = bc_internal;
+    hd->sub_class.id = sc_int_cpu;
+    hd->slot = 0;
+    hd->detail = new_mem(sizeof *hd->detail);
+    hd->detail->type = hd_detail_cpu;
+    hd->detail->cpu.data = ct;
+  }
+#endif	/* __m68k__ */
 }
 
 /*
