@@ -19,7 +19,7 @@ extern "C" {
  */
 
 /** Interface version */
-#define HD_VERSION	15
+#define HD_VERSION	21
 
 /**
  * @defgroup DEBUGpub Debug flags
@@ -59,6 +59,8 @@ extern "C" {
 #include <termios.h>
 #include <sys/types.h>
 
+//typedef struct vm_s vm_t;
+
 /**
  * libhd's directory
  */
@@ -117,8 +119,8 @@ typedef enum probe_feature {
   pr_block, pr_block_cdrom, pr_block_part, pr_edd, pr_edd_mod, pr_bios_ddc,
   pr_bios_fb, pr_bios_mode, pr_input, pr_block_mods, pr_bios_vesa,
   pr_cpuemu_debug, pr_scsi_noserial, pr_wlan, pr_bios_crc, pr_hal,
-  pr_bios_vram, pr_bios_acpi, pr_bios_ddc_ports_1, pr_bios_ddc_ports_2,
-  pr_bios_ddc_ports_3, pr_bios_ddc_ports_4, pr_modules_pata,
+  pr_bios_vram, pr_bios_acpi, pr_bios_ddc_ports, pr_modules_pata,
+  pr_net_eeprom, pr_x86emu,
   pr_max, pr_lxrc, pr_default, 
   pr_all		/**< pr_all must be last */
 } hd_probe_feature_t;
@@ -211,7 +213,7 @@ typedef enum sc_input {
 /** subclass values of bc_serial */
 typedef enum sc_serial {
   sc_ser_fire, sc_ser_access, sc_ser_ssa, sc_ser_usb, sc_ser_fiber,
-  sc_ser_smbus, sc_ser_other = 0x80
+  sc_ser_smbus, sc_ser_infiniband, sc_ser_other = 0x80
 } hd_sc_serial_t;
 
 /** internal sub class values (bc_internal) */
@@ -294,7 +296,7 @@ typedef enum bus_types {
   /** outside the range of the PCI values */
   bus_ps2 = 0x80, bus_serial, bus_parallel, bus_floppy, bus_scsi, bus_ide, bus_usb,
   bus_adb, bus_raid, bus_sbus, bus_i2o, bus_vio, bus_ccw, bus_iucv, bus_ps3_system_bus,
-  bus_virtio
+  bus_virtio, bus_ibmebus, bus_gameport, bus_uisvirtpci
 } hd_bus_types_t;
 
 /** @} */
@@ -1045,6 +1047,7 @@ typedef struct s_pci_t {
   char *sysfs_id;				/**< sysfs path */
   char *sysfs_bus_id;				/**< sysfs bus id */
   char *modalias;				/**< module alias */
+  char *label;					/**< Consistant Device Name (CDN), pci firmware spec 3.1, chapter 4.6.7 */
   unsigned edid_len[4];				/**< edid record length */
   unsigned char edid_data[4][0x80];		/**< edid record */
 } pci_t;
@@ -1271,6 +1274,13 @@ typedef struct ccw_s {
   unsigned char dev_model;
 } ccw_t;
 
+/**
+ * @brief Joystick details
+ */
+typedef struct joystick_s {
+  unsigned char buttons;
+  unsigned char axes;
+} joystick_t;
 
 /**
  * @brief special CDROM entry
@@ -1428,7 +1438,8 @@ typedef enum cpu_arch {
   arch_s390, arch_s390x,
   arch_arm,
   arch_mips,
-  arch_x86_64
+  arch_x86_64,
+  arch_aarch64
 } hd_cpu_arch_t;
 
 /**
@@ -1437,7 +1448,7 @@ typedef enum cpu_arch {
 typedef enum boot_arch {
   boot_unknown = 0,
   boot_lilo, boot_milo, boot_aboot, boot_silo, boot_ppc, boot_elilo, boot_s390,
-  boot_mips, boot_grub
+  boot_mips, boot_grub, boot_uboot,
 } hd_boot_arch_t;
 
 
@@ -1483,6 +1494,8 @@ typedef struct {
   unsigned ext_64bit:1;
   unsigned assigned:1;
   unsigned valid:1;
+  unsigned ext_fibre:1;
+  unsigned ext_net:1;
   char *sysfs_id;
   unsigned hd_idx;
   unsigned signature;
@@ -1612,7 +1625,6 @@ typedef struct hal_device_s {
   hal_prop_t *prop;
 } hal_device_t;
 
-
 /**
  * resource types: see @ref RESOURCEpub
  */
@@ -2049,7 +2061,7 @@ typedef enum hd_detail_type {
   hd_detail_pci, hd_detail_usb, hd_detail_isapnp, hd_detail_cdrom,
   hd_detail_floppy, hd_detail_bios, hd_detail_cpu, hd_detail_prom,
   hd_detail_monitor, hd_detail_sys, hd_detail_scsi, hd_detail_devtree,
-  hd_detail_ccw
+  hd_detail_ccw, hd_detail_joystick
 } hd_detail_type_t;
 
 typedef struct {
@@ -2118,6 +2130,11 @@ typedef struct {
   ccw_t *data;
 } hd_detail_ccw_t;
 
+typedef struct {
+  enum hd_detail_type type;
+  joystick_t *data;
+} hd_detail_joystick_t;
+
 typedef union {
   enum hd_detail_type type;
   hd_detail_pci_t pci;
@@ -2133,6 +2150,7 @@ typedef union {
   hd_detail_scsi_t scsi;
   hd_detail_devtree_t devtree;
   hd_detail_ccw_t ccw;
+  hd_detail_joystick_t joystick;
 } hd_detail_t;
 
 /** @} */
@@ -2457,21 +2475,39 @@ typedef struct s_hd_t {
     unsigned dvd:1;		/**< DVD */
     unsigned dvdr:1;		/**< DVD-R */
     unsigned dvdrw:1;		/**< DVD-RW */
+    unsigned dvdrdl:1;		/**< DVD-R DL */
     unsigned dvdpr:1;		/**< DVD+R */
     unsigned dvdprw:1;		/**< DVD+RW */
-    unsigned dvdprdl:1;		/**< DVD+RDL */
+    unsigned dvdprdl:1;		/**< DVD+R DL */
+    unsigned dvdprwdl:1;	/**< DVD+RW DL */
+    unsigned bd:1;		/**< BD */
+    unsigned bdr:1;		/**< BD-R */
+    unsigned bdre:1;		/**< BD-RE */
+    unsigned hd:1;		/**< HD */
+    unsigned hdr:1;		/**< HD-R */
+    unsigned hdrw:1;		/**< HD-RW */
     unsigned dvdram:1;		/**< DVDRAM */
+    unsigned mo:1;		/**< MO */
+    unsigned mrw:1;		/**< MRW */
+    unsigned mrww:1;		/**< MRW-W */
     unsigned pppoe:1;		/**< PPPOE modem connected */
     unsigned wlan:1;		/**< WLAN card */
     unsigned with_acpi:1;	/**< acpi works fine */
     unsigned hotpluggable:1;	/**< hotpluggable storage device */
+    unsigned dualport:1;	/**< OSA Express device with two ports (S/390) */
+    unsigned fcoe:1;		/**< fcoe device */
+    unsigned fcoe_offload:2;	/**< fcoe offload capable device */
+    unsigned iscsi_offload:2;	/**< iscsi offload capable  device */
+    unsigned storage_only:2;	/**< storage only network interface */
   } is;
 
   struct tag_s {		/**< this struct is for internal purposes only */
     unsigned remove:1;		/**< schedule for removal */
     unsigned freeit:1;		/**< for internal memory management */
     unsigned fixed:1;		/**< fixed, do no longer modify this entry */
-    unsigned ser_skip:1;	/**< if serial line, don't scan for devices */
+    unsigned skip_mouse:1;	/**< if serial line, don't scan for mice */
+    unsigned skip_modem:1;	/**< if serial line, don't scan for modems */
+    unsigned skip_braille:1;	/**< if serial line, don't scan for braille devices */
     unsigned ser_device:2;	/**< if != 0: info about attached serial device; see serial.c */
   } tag;
 
@@ -2548,6 +2584,7 @@ typedef struct s_hd_t {
   hal_prop_t *persistent_prop;	/**< persistent property list */
 
   char *modalias;		/**< module alias */
+  char *label;			/**< Consistent Device Name (CDN), pci firmware spec 3.1, chapter 4.6.7 */
 
   /*
    * These are used internally for memory management.
@@ -2636,6 +2673,7 @@ typedef struct {
   unsigned char probe[(pr_all + 7) / 8];	/**< (Internal) bitmask of probing features. */
   unsigned char probe_set[(pr_all + 7) / 8];	/**< (Iternal) bitmask of probing features that will always be set. */
   unsigned char probe_clr[(pr_all + 7) / 8];	/**< (Internal) bitmask of probing features that will always be reset. */
+  hal_prop_t *probe_val;	/**< (Internal) probing features with arbitrary values */
   unsigned last_idx;		/**< (Internal) index of the last hd entry generated */
   unsigned module;		/**< (Internal) the current probing module we are in */
   enum boot_arch boot;		/**< (Internal) boot method */
@@ -2690,6 +2728,10 @@ typedef struct {
   edd_info_t edd[0x80];		/**< (Internal) enhanced disk drive data */
   hal_device_t *hal;		/**< (Internal) HAL data (if any) */
   str_list_t *lsscsi;		/**< (Internal) lsscsi result (if any) */
+  struct vm_s *vm;		/**< (Internal) x86emu vm */
+  size_t log_size;		/**< (Internal) current log size (including final 0) */
+  size_t log_max;		/**< (Internal) log buffer size */
+  str_list_t *klog_raw;		/**< (Internal) unmodified kernel log */
 } hd_data_t;
 
 
