@@ -52,13 +52,6 @@ void hd_dump_entry(hd_data_t *hd_data, hd_t *h, FILE *f)
   str_list_t *sl;
   hal_prop_t *prop;
 
-#ifdef LIBHD_MEMCHECK
-  {
-    if(libhd_log)
-      fprintf(libhd_log, "; %s\t%p\t%p\n", __FUNCTION__, CALLED_FROM(hd_dump_entry, hd_data), hd_data);
-  }
-#endif
-
   if(!h) return;
 
   s = "";
@@ -364,6 +357,8 @@ void dump_normal(hd_data_t *hd_data, hd_t *h, FILE *f)
   hd_detail_monitor_t *mdetail;
   static char *geo_type_str[] = { "Physical", "Logical", "BIOS EDD", "BIOS Legacy" };
 
+  if(h->label) dump_line("Device Name: \"%s\"\n", h->label);
+
   if(h->model) dump_line("Model: \"%s\"\n", h->model);
 
   s = NULL;
@@ -502,19 +497,25 @@ void dump_normal(hd_data_t *hd_data, hd_t *h, FILE *f)
     dump_line("PROM id: %s\n", h->rom_id);
 #endif
 #if defined(__s390__) || defined(__s390x__)
-    dump_line("IUCV user: %s\n", h->rom_id);
+    dump_line("IUCV user: %s", h->rom_id);
 #endif
   }
 
-  if(h->tag.ser_skip) {
-    dump_line_str("Tags: ser_skip\n");
+  if(h->tag.skip_mouse || h->tag.skip_modem || h->tag.skip_braille) {
+    dump_line_str("Tags:");
+    i = 0;
+    if(h->tag.skip_mouse) dump_line0("%s mouse", i++ ? "," : "");
+    if(h->tag.skip_modem) dump_line0("%s modem", i++ ? "," : "");
+    if(h->tag.skip_braille) dump_line0("%s braille", i++ ? "," : "");
+    dump_line0("\n");
   }
 
   if(
     h->is.zip ||
     h->is.cdr || h->is.cdrw || h->is.dvd || h->is.dvdr || h->is.dvdrw ||
     h->is.dvdpr || h->is.dvdprw || h->is.dvdprdl || h->is.dvdram ||
-    h->is.pppoe || h->is.wlan || h->is.hotpluggable
+    h->is.pppoe || h->is.wlan || h->is.hotpluggable || h->is.fcoe ||
+    h->is.fcoe_offload || h->is.iscsi_offload || h->is.storage_only
   ) {
     dump_line_str("Features:");
     i = 0;
@@ -524,12 +525,27 @@ void dump_normal(hd_data_t *hd_data, hd_t *h, FILE *f)
     if(h->is.dvd) dump_line0("%s DVD", i++ ? "," : "");
     if(h->is.dvdr) dump_line0("%s DVD-R", i++ ? "," : "");
     if(h->is.dvdrw) dump_line0("%s DVD-RW", i++ ? "," : "");
+    if(h->is.dvdrdl) dump_line0("%s DVD-R DL", i++ ? "," : "");
     if(h->is.dvdpr) dump_line0("%s DVD+R", i++ ? "," : "");
     if(h->is.dvdprw) dump_line0("%s DVD+RW", i++ ? "," : "");
-    if(h->is.dvdprdl) dump_line0("%s DVD+DL", i++ ? "," : "");
-    if(h->is.dvdram) dump_line0("%s DVDRAM", i++ ? "," : "");
+    if(h->is.dvdprdl) dump_line0("%s DVD+R DL", i++ ? "," : "");
+    if(h->is.dvdprwdl) dump_line0("%s DVD+RW DL", i++ ? "," : "");
+    if(h->is.bd) dump_line0("%s BD", i++ ? "," : "");
+    if(h->is.bdr) dump_line0("%s BD-R", i++ ? "," : "");
+    if(h->is.bdre) dump_line0("%s BD-RE", i++ ? "," : "");
+    if(h->is.hd) dump_line0("%s HD", i++ ? "," : "");
+    if(h->is.hdr) dump_line0("%s HD-R", i++ ? "," : "");
+    if(h->is.hdrw) dump_line0("%s HD-RW", i++ ? "," : "");
+    if(h->is.dvdram) dump_line0("%s DVD-RAM", i++ ? "," : "");
+    if(h->is.mo) dump_line0("%s MO", i++ ? "," : "");
+    if(h->is.mrw) dump_line0("%s MRW", i++ ? "," : "");
+    if(h->is.mrww) dump_line0("%s MRW-W", i++ ? "," : "");
     if(h->is.pppoe) dump_line0("%s PPPOE", i++ ? "," : "");
     if(h->is.wlan) dump_line0("%s WLAN", i++ ? "," : "");
+    if(h->is.fcoe) dump_line0("%s FCoE", i++ ? "," : "");
+    if(h->is.fcoe_offload) dump_line0("%s FCoEOffload = %s", i++ ? "," : "", h->is.fcoe_offload == 1 ? "off" : "on");
+    if(h->is.iscsi_offload) dump_line0("%s iSCSIOffload = %s", i++ ? "," : "", h->is.iscsi_offload == 1 ? "off" : "on");
+    if(h->is.storage_only) dump_line0("%s StorageOnly = %s", i++ ? "," : "", h->is.storage_only == 1 ? "off" : "on");
     if(h->is.hotpluggable) dump_line0("%s Hotpluggable", i++ ? "," : "");
     dump_line0("\n");
   }
@@ -653,12 +669,20 @@ void dump_normal(hd_data_t *hd_data, hd_t *h, FILE *f)
               snprintf(b0, sizeof b0 - 1, "%"PRIu64, res->size.val1);
               snprintf(b1, sizeof b1 - 1, "%"PRIu64, res->size.val2);
           }
-          if(!res->size.val2)
+          if(!res->size.val2) {
             dump_line("Size: %s %s\n", b0, s);
-          else if(res->size.unit == size_unit_sectors)
+          }
+          else if(res->size.unit == size_unit_sectors) {
             dump_line("Size: %s %s a %s bytes\n", b0, s, b1);
-          else
+	    dump_line(
+	      "Capacity: %" PRIu64 " GB (%" PRIu64 " bytes)\n",
+	      (res->size.val1 * res->size.val2) >> 30,
+	      res->size.val1 * res->size.val2
+	    );
+	  }
+          else {
             dump_line("Size: %sx%s %s\n", b0, b1, s);
+          }
         }
         break;
 
@@ -793,6 +817,15 @@ void dump_normal(hd_data_t *hd_data, hd_t *h, FILE *f)
     dump_line("Module Alias: \"%s\"\n", h->modalias);
   }
 
+  // print joystick details
+  if(h->detail && h->detail->type == hd_detail_joystick)
+  {
+    joystick_t *jt = h->detail->joystick.data;
+
+    dump_line("Buttons: %hhd\n", jt->buttons);
+    dump_line("Axes: %hhd\n", jt->axes);
+  }
+
   if(
     h->detail &&
     h->detail->type == hd_detail_monitor &&
@@ -808,13 +841,13 @@ void dump_normal(hd_data_t *hd_data, hd_t *h, FILE *f)
           "   Horizontal: %4u %4u %4u %4u (+%u +%u +%u) %chsync\n",
           mi->hdisp, mi->hsyncstart, mi->hsyncend, mi->htotal,
           mi->hsyncstart - mi->hdisp, mi->hsyncend - mi->hdisp, mi->htotal - mi->hdisp,
-          mi->hflag
+          mi->hflag ?: ' '
         );
         dump_line(
           "     Vertical: %4u %4u %4u %4u (+%u +%u +%u) %cvsync\n",
           mi->vdisp, mi->vsyncstart, mi->vsyncend, mi->vtotal,
           mi->vsyncstart - mi->vdisp, mi->vsyncend - mi->vdisp, mi->vtotal - mi->vdisp,
-          mi->vflag
+          mi->vflag ?: ' '
         );
         dump_line(
           "  Frequencies: %.2f MHz, %.2f kHz, %.2f Hz\n",
@@ -1069,6 +1102,9 @@ void dump_cpu(hd_data_t *hd_data, hd_t *hd, FILE *f)
       case arch_x86_64:
 	dump_line0 ("X86-64\n");
 	break;
+      case arch_aarch64:
+	dump_line0 ("AArch64\n");
+	break;
       default:
 	dump_line0 ("**UNKNWON**\n");
 	break;
