@@ -80,6 +80,9 @@ void hd_scan_int(hd_data_t *hd_data)
   PROGRESS(6, 0, "mouse");
   int_mouse(hd_data);
 
+  /* data already needed in int_system() */
+  hd_sysfs_driver_list(hd_data);
+
 #if defined(__i386__) || defined (__x86_64__)
   PROGRESS(15, 0, "system info");
   int_system(hd_data);
@@ -1082,6 +1085,8 @@ void int_system(hd_data_t *hd_data)
   } is = { };
   char *s;
   struct stat sbuf;
+  sys_info_t *st;
+  hd_sysfsdrv_t *sf;
 
   for(hd_sys = hd_data->hd; hd_sys; hd_sys = hd_sys->next) {
     if(
@@ -1092,6 +1097,8 @@ void int_system(hd_data_t *hd_data)
 
   if(!hd_sys) return;
 
+  st = hd_sys->detail && hd_sys->detail->type == hd_detail_sys ? hd_sys->detail->sys.data : NULL;
+
   if(
     hd_sys->vendor.name &&
     !strncasecmp(hd_sys->vendor.name, "sony", sizeof "sony" - 1)
@@ -1160,6 +1167,28 @@ void int_system(hd_data_t *hd_data)
     ) {
       is.notebook = 1;
     }
+
+    /*
+     * bnc #591703
+     * in case chassis info is missing: assume it's a notebook if
+     * it has track point or touch pad
+     */
+    if(
+      sm->any.type == sm_mouse &&
+      (sm->mouse.mtype.id == 5 || sm->mouse.mtype.id == 7)
+    ) {
+      is.notebook = 1;
+    }
+  }
+
+  /* check for battery, too (bnc #678456) */
+  if(!is.notebook) {
+    for(sf = hd_data->sysfsdrv; sf; sf = sf->next) {
+      if(sf->driver && !strcmp(sf->driver, "battery")) {
+        is.notebook = 1;
+        break;
+      }
+    }
   }
 
   ADD2LOG(
@@ -1171,6 +1200,10 @@ void int_system(hd_data_t *hd_data)
     is.notebook ? " notebook" : ""
   );
 
+  if(st && !st->formfactor) {
+    st->formfactor = new_str(is.notebook ? "laptop" : "desktop");
+  }
+
   if(is.notebook && is.vendor) {
     hd_sys->compat_vendor.id = MAKE_ID(TAG_SPECIAL, 0xf001);
     hd_sys->compat_device.id = MAKE_ID(TAG_SPECIAL, is.vendor);
@@ -1249,8 +1282,6 @@ void int_add_driver_modules(hd_data_t *hd_data)
 {
   hd_t *hd;
 
-  hd_sysfs_driver_list(hd_data);
-
   for(hd = hd_data->hd; hd; hd = hd->next) {
     int_update_driver_data(hd_data, hd);
   }
