@@ -58,6 +58,7 @@ static void hd_read_of_platform(hd_data_t *hd_data);
 static void add_xen_network(hd_data_t *hd_data);
 static void add_xen_storage(hd_data_t *hd_data);
 static void hd_read_virtio(hd_data_t *hd_data);
+static void hd_read_uisvirtpci(hd_data_t *hd_data);
 static void hd_read_ibmebus(hd_data_t *hd_data);
 
 void hd_scan_sysfs_pci(hd_data_t *hd_data)
@@ -107,6 +108,9 @@ void hd_scan_sysfs_pci(hd_data_t *hd_data)
 
   PROGRESS(11, 0, "ibmebus");
   hd_read_ibmebus(hd_data);
+
+  PROGRESS(12, 0, "uisvirtpci");
+  hd_read_uisvirtpci(hd_data);
 }
 
 
@@ -192,6 +196,11 @@ void hd_pci_read_data(hd_data_t *hd_data)
       pci->irq = ul0;
     }
 
+    if((s = get_sysfs_attr_by_path(sf_dev, "label"))) {
+      pci->label = canon_str(s, strlen(s));
+      ADD2LOG("    label = \"%s\"\n", pci->label);
+    }
+
     sl = hd_attr_list(get_sysfs_attr_by_path(sf_dev, "resource"));
     for(u = 0; sl; sl = sl->next, u++) {
       if(
@@ -270,7 +279,7 @@ void hd_pci_read_data(hd_data_t *hd_data)
         if(pci->edid_len[u] > 0) {
           for(i = 0; i < sizeof pci->edid_data[u]; i += 0x10) {
             ADD2LOG("      ");
-            hexdump(&hd_data->log, 1, 0x10, pci->edid_data[u] + i);
+            hd_log_hex(hd_data, 1, 0x10, pci->edid_data[u] + i);
             ADD2LOG("\n");
           }
         }
@@ -310,6 +319,7 @@ void add_pci_data(hd_data_t *hd_data)
   pci_t *pci, *pnext;
   unsigned u;
   char *s, *t;
+  str_list_t *net_ifs, *net_ifs2;
 
   PROGRESS(4, 0, "build list");
 
@@ -333,6 +343,58 @@ void add_pci_data(hd_data_t *hd_data)
       hd->base_class.id = u >> 8;
       hd->sub_class.id = u & 0xff;
     }
+
+    /*
+     * If there are several network interfaces attached to a single
+     * function, just replicate entries.
+     */
+
+    /* old kernels: subdir 'net/<interface>' */
+    s = NULL;
+    str_printf(&s, 0, "/sys%s/net", hd->sysfs_id);
+    net_ifs = read_dir(s, 'D');
+    s = free_mem(s);
+
+    if(!net_ifs) {
+      /* old kernels: links 'net:<interface>' */
+      str_list_t *tmp, *sl;
+
+      str_printf(&s, 0, "/sys%s", hd->sysfs_id);
+      tmp = read_dir(s, 'l');
+      s = free_mem(s);
+
+      for(sl = tmp; sl; sl = sl->next) {
+        if(!strncmp(sl->str, "net:", sizeof "net:" - 1)) {
+          add_str_list(&net_ifs, sl->str + sizeof "net:" - 1);
+        }
+      }
+
+      free_str_list(tmp);
+    }
+
+    if(net_ifs) {
+      hd->unix_dev_name = new_str(net_ifs->str);
+      net_ifs2 = net_ifs->next;
+
+      for(; net_ifs2; net_ifs2 = net_ifs2->next) {
+        hd2 = add_hd_entry(hd_data, __LINE__, 0);
+        hd2->sysfs_id = new_str(hd->sysfs_id);
+        if(hd->drivers) {
+          add_str_list(&hd2->drivers, hd->drivers->str);
+        }
+        hd2->unix_dev_name = new_str(net_ifs2->str);
+        hd2->detail = hd->detail;
+        hd_pci_complete_data(hd2);
+        hd2->detail = NULL;
+
+        if((u = device_class(hd_data, hd2->vendor.id, hd2->device.id))) {
+          hd2->base_class.id = u >> 8;
+          hd2->sub_class.id = u & 0xff;
+        }
+      }
+    }
+
+    free_str_list(net_ifs);
   }
 
   hd_data->pci = NULL;
@@ -379,6 +441,11 @@ void hd_pci_complete_data(hd_t *hd)
     pci->modalias = NULL;
   }
 
+  if(pci->label && *pci->label) {
+    hd->label = pci->label;
+    pci->label = NULL;
+  }
+
   hd->slot = pci->slot + (pci->bus << 8);
   hd->func = pci->func;
   hd->base_class.id = pci->base_class;
@@ -563,7 +630,7 @@ void dump_pci_data(hd_data_t *hd_data)
     for(i = 0; (unsigned) i < pci->data_ext_len; i += 0x10) {
       ADD2LOG("  %02x: ", i);
       j = pci->data_ext_len - i;
-      hexdump(&hd_data->log, 1, j > 0x10 ? 0x10 : j, pci->data + i);
+      hd_log_hex(hd_data, 1, j > 0x10 ? 0x10 : j, pci->data + i);
       ADD2LOG("\n");
     }
 
@@ -727,6 +794,7 @@ void hd_read_vio(hd_data_t *hd_data)
         !strcmp(vio_name, "l-lan") || /* pseries && iseries */
         !strcmp(vio_name, "viodasd") || /* iseries */
         !strcmp(vio_name, "viocd") || /* iseries */
+        !strcmp(vio_name, "vfc-client") || /* ibmvfc */
         !strcmp(vio_name, "v-scsi" /* pseries */)
       )
     ) {
@@ -751,6 +819,11 @@ void hd_read_vio(hd_data_t *hd_data)
         hd->sub_class.id = sc_sto_other;
         hd->slot = cd_cnt++;
         str_printf(&hd->device.name, 0, "Virtual CD %d", hd->slot);
+      } else if(!strcmp(vio_name, "vfc-client")) {
+        hd->base_class.id = bc_storage;
+        hd->sub_class.id = sc_sto_scsi;
+        hd->slot = scsi_cnt++;
+        str_printf(&hd->device.name, 0, "Virtual FC %d", hd->slot);
       }
       else { /* scsi */
         hd->base_class.id = bc_storage;
@@ -1325,7 +1398,7 @@ void add_xen_storage(hd_data_t *hd_data)
  */
 void hd_read_vm(hd_data_t *hd_data)
 {
-  int eth_cnt = 0;
+  int eth_cnt = 0, blk_cnt = 0;
   hd_t *hd;
   str_list_t *sf_bus, *sf_bus_e;
   char *sf_dev, *drv, *drv_name;
@@ -1357,7 +1430,7 @@ void hd_read_vm(hd_data_t *hd_data)
 
     if(
       drv_name &&
-      !strcmp(drv_name, "netvsc")
+      !strcmp(drv_name, "hv_netvsc")
     ) {
       hd = add_hd_entry(hd_data, __LINE__, 0);
       hd->bus.id = bus_none;
@@ -1374,6 +1447,25 @@ void hd_read_vm(hd_data_t *hd_data)
       hd->sysfs_bus_id = new_str(sf_bus_e->str);
       if(drv_name) add_str_list(&hd->drivers, drv_name);
     }
+    else if(
+      drv_name &&
+      !strcmp(drv_name, "hv_storvsc")
+    ) {
+      hd = add_hd_entry(hd_data, __LINE__, 0);
+      hd->bus.id = bus_none;
+
+      hd->vendor.id = MAKE_ID(TAG_SPECIAL, 0x6013);	/* virtual */
+
+      hd->base_class.id = bc_storage;
+      hd->sub_class.id = sc_sto_other;
+      hd->slot = blk_cnt++;
+      hd->device.id = MAKE_ID(TAG_SPECIAL, 2);
+      str_printf(&hd->device.name, 0, "Storage %d", hd->slot);
+
+      hd->sysfs_id = new_str(hd_sysfs_id(sf_dev));
+      hd->sysfs_bus_id = new_str(sf_bus_e->str);
+      if(drv_name) add_str_list(&hd->drivers, drv_name);
+    }
 
     free_mem(sf_dev);
     free_mem(drv);
@@ -1471,6 +1563,76 @@ void hd_read_virtio(hd_data_t *hd_data)
   free_str_list(sf_bus);
 }
 
+/*
+ * uisvirtpci
+ */
+void hd_read_uisvirtpci(hd_data_t *hd_data)
+{
+  uint64_t ul0;
+  hd_t *hd;
+  str_list_t *sf_bus, *sf_bus_e;
+  char *sf_dev, *drv, *drv_name;
 
+  sf_bus = read_dir("/sys/bus/uisvirtpci/devices", 'l');
+
+  if(!sf_bus) {
+    ADD2LOG("sysfs: no such bus: uisvirtpci\n");
+    return;
+  }
+
+  for(sf_bus_e = sf_bus; sf_bus_e; sf_bus_e = sf_bus_e->next) {
+    sf_dev = new_str(hd_read_sysfs_link("/sys/bus/uisvirtpci/devices", sf_bus_e->str));
+
+    ADD2LOG(
+      "  uisvirtpci device: name = %s\n    path = %s\n",
+      sf_bus_e->str,
+      hd_sysfs_id(sf_dev)
+    );
+
+    hd = add_hd_entry(hd_data, __LINE__, 0);
+    hd->vendor.id = MAKE_ID(TAG_PCI, 0xA0F1);	/* Unisys */
+
+    drv_name = NULL;
+    drv = new_str(hd_read_sysfs_link(sf_dev, "driver"));
+    if(drv) {
+        drv_name = strrchr(drv, '/');
+        if(drv_name) {
+            drv_name++;
+            ADD2LOG("    driver = \"%s\"\n", drv_name);
+
+            if(!strcmp(drv_name,"uisvirtnic")) {
+              hd->device.id = MAKE_ID(TAG_SPECIAL, 0x0002);
+              hd->base_class.id = bc_network;
+            }
+            else if(!strcmp(drv_name,"uisvirthba")) {
+              hd->device.id = MAKE_ID(TAG_SPECIAL, 0x0003);
+              hd->base_class.id = bc_storage;
+            }
+            else {
+              hd->device.id = MAKE_ID(TAG_SPECIAL, 0x0000);
+              hd->base_class.id = bc_other;
+            }
+        }
+    }
+    else {
+        hd->device.id = MAKE_ID(TAG_SPECIAL, 0x0001);
+        hd->base_class.id = bc_bridge;
+    }
+
+   if(hd_attr_uint(get_sysfs_attr_by_path(sf_dev, "device"), &ul0, 0)) {
+       hd->device.id = MAKE_ID(TAG_SPECIAL, ul0 );
+       ADD2LOG("    device = %lu\n", ul0);
+    }
+
+    hd->sysfs_id = new_str(hd_sysfs_id(sf_dev));
+    hd->sysfs_bus_id = new_str(sf_bus_e->str);
+    if(drv_name) add_str_list(&hd->drivers, drv_name);
+
+    free_mem(sf_dev);
+    free_mem(drv);
+  }
+
+  free_str_list(sf_bus);
+}
 /** @} */
 
